#include "tsmuxerwindow.h"
#include "lang_codes.h"
#include <QApplication>
#include <QUrl>

#ifdef WIN32
#include "windows.h"
#endif

const char fileDialogFilter[] = 
"All supported media files (*.aac *.mpv *.mpa *.avc *.264 *.h264 *.ac3 *.dts *.ts *.m2ts *.mpg *.vob *.evo *.mkv *.mka *.mks *.sup *.wav *.w64 *.pcm *.m1v *.m2v *.vc1 *.mpls *.srt);;\
AC3/E-AC3 (*.ac3 *.ddp);;\
AAC (advanced audio coding) (*.aac);;\
AVC/H.264 elementary stream (*.avc *.264 *.h264);;\
Digital Theater System (*.dts);;\
Mpeg video elementary stream (*.mpv *.m1v *.m2v);;\
Mpeg audio elementary stream (*.mpa);;\
Transport Stream (*.ts);;\
BDAV Transport Stream (*.m2ts *.mts);;\
Program Stream (*.mpg *.vob *.evo);;\
Matroska audio/video files (*.mkv *.mka *.mks);;\
Blu-ray play list (*.mpls);;\
Blu-ray PGS subtitles (*.sup);;\
Text subtitles (*.srt);;\
WAVE - Uncompressed PCM audio (*.wav *.w64);;\
RAW LPCM Stream (*.pcm);;\
All files (*.*)";
const char saveMetaFilter[] = "tsMuxeR project file (*.meta);;All files (*.*)";

const double EPS = 1e-6;
const char TI_DEFAULT_TAB_NAME[] = "General track options";
const char TI_DEMUX_TAB_NAME[] = "Demux options";
const char TS_SAVE_DIALOG_FILTER[] = "Transport stream (*.ts);;all files (*.*)";
const char M2TS_SAVE_DIALOG_FILTER[] = "BDAV Transport Stream (*.m2ts);;all files (*.*)";

QLocale locale;

QString unquoteStr(QString val)
{
	val = val.trimmed();
	if (val.isEmpty())
		return val;
	if (val.at(0) == '\"') {
		if (val.right(1) == "\"")
			return val.mid(1, val.length()-2);
		else
			return val.mid(1, val.length()-1);
	}
	else {
		if (val.right(1) == "\"")
			return val.mid(0, val.length()-1);
		else
			return val;
	}
}

QString floatToTime(double time, char msSeparator)
{
	int iTime = (int) time;
	int hour =iTime / 3600;
	iTime -= hour*3600;
	int min =iTime / 60;
	iTime -= min*60;
	int sec =iTime;
	int msec = (int) ((time - (int)time)*1000.0);
	QString str;
	str += QString::number(hour).rightJustified(2,'0');
	str +=':';
	str += QString::number(min).rightJustified(2,'0');
	str +=':';
	str += QString::number(sec).rightJustified(2,'0');
	str += msSeparator;
	str += QString::number(msec).rightJustified(3,'0');
	return str;
}

QString changeFileExt(const QString& value, const QString& newExt)
{
	QFileInfo fi(unquoteStr(value));
	if (fi.suffix().length() > 0 || (!value.isEmpty() && value.right(1) == "."))
		return unquoteStr(value.left(value.length() - fi.suffix().length()) + newExt);
	else
		return unquoteStr(value) + "." + newExt;
}

QString toFixedDecPoint(QString str)
{
	for (int i = 0; i < str.length(); ++i)
		if (str[i] == ',')
			str[i] = '.';
	return str;
}

QString toNativeDecPoint(QString str)
{
	for (int i = 0; i < str.length(); ++i)
		if (str[i] == ',' || str[i] == '.')
			str[i] = locale.decimalPoint();
	return str;
}

float myStrToFloat(QString str)
{
	for (int i = 0; i < str.length(); ++i)
		if (str[i] == ',' || str[i] == '.')
			str[i] = locale.decimalPoint();
	return str.toFloat();
}

QString myFloatToStr(float val)
{
	QString str = QString::number(val);
	for (int i = 0; i < str.length(); ++i)
		if (str[i] == locale.decimalPoint())
			str[i] = '.';
	return str;
}

QString fpsTextToFpsStr(const QString& fpsText)
{
	int p = fpsText.indexOf('/');
	if (p >= 0) {
		QString leftStr  = fpsText.mid(0, p);
		QString rightStr = fpsText.mid(p+1);
		return toFixedDecPoint(QString::number(myStrToFloat(leftStr) / myStrToFloat(rightStr), 'f', 3));
	}
	else
		return myFloatToStr(myStrToFloat(fpsText));
}

float extractFloatFromDescr(const QString& str, const QString& pattern)
{
	int p = 0;
	try {
		if (!pattern.isEmpty()) 
			p = str.indexOf(pattern);
		if (p >= 0) {
			p += pattern.length();
			int p2 = p;
			while (p2 < str.length() && (str.at(p2) >= '0' && str.at(p2) <= '9' || str.at(p2) == '.' || str.at(p2) == '-'))
				p2++;
			QString tmp = str.mid(p, p2-p);
			return myStrToFloat(tmp);
		}
	} catch(...) {
		return 0;
	}
	return 0;
}


QString quoteStr(const QString& val)
{
	QString rez;
	if(val.isEmpty())
		return "";
	if (val.at(0) != '\"')
		rez = '\"' + val;
	else
		rez = val;
	if (val.at(val.length()-1) != '\"') 
		rez += '\"';
	return rez;
}

QString myUnquoteStr(const QString& val)
{
	QString trimmedVal = val.trimmed();
	int len = trimmedVal.length();
	for (int i = 0; i < len; ++i) {
		if (trimmedVal.at(i) != ' ' && trimmedVal.at(i) != '+' && trimmedVal.at(i) != '\"')  
		{
			if (trimmedVal.at(len-1) == '\"')
				return trimmedVal.mid(i, len - i - 1); // todo: check here
			else
				return trimmedVal.mid(i, len - i);
		}
	}
	return QString();
}

//////////////////////////////////////////////

TsMuxerWindow::TsMuxerWindow(): 
	trackLVIngoreOnChange(false),
	outFileNameModified(false),
	outFileNameDisableChange(false),
	muxForm(this),
	tempSoundFile(0),
	sound(0)
{
	ui.setupUi(this);
	// next properties supported by Designer in version 4.5 only.
	ui.listViewFont->horizontalHeader()->setVisible(false);
	ui.listViewFont->verticalHeader()->setVisible(false);
	ui.listViewFont->horizontalHeader()->setStretchLastSection(true);
	ui.trackLV->horizontalHeader()->setStretchLastSection(true);
	/////////////////////////////////////////////////////////////
	for (int i = 0; i <= 3600; i+= 5*60)
		ui.memoChapters->insertPlainText(floatToTime(i,'.') + '\n');

	saveDialogFilter = TS_SAVE_DIALOG_FILTER;
	const static int colWidths[] = 
	{
		28, 200, 62, 38, 10
	};
	for (int i = 0; i < sizeof(colWidths) / sizeof(int); ++i)
		ui.trackLV->horizontalHeader()->resizeSection(i, colWidths[i]);

	ui.listViewFont->horizontalHeader()->resizeSection(0, 65);
	ui.listViewFont->horizontalHeader()->resizeSection(1, 185);
	for (int i = 0; i < ui.listViewFont->rowCount(); ++i) {
		ui.listViewFont->setRowHeight(i, 16);
		ui.listViewFont->item(i,0)->setFlags(ui.listViewFont->item(i,0)->flags() & (~Qt::ItemIsEditable));
		ui.listViewFont->item(i,1)->setFlags(ui.listViewFont->item(i,0)->flags() & (~Qt::ItemIsEditable));
	}
	connect(&opacityTimer,			   SIGNAL(timeout()), this, SLOT(onOpacityTimer()));
	connect(ui.trackLV,                SIGNAL(itemSelectionChanged()), this, SLOT(trackLVItemSelectionChanged()));
	connect(ui.trackLV,                SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(trackLVItemChanged(QTableWidgetItem*)));
	connect(ui.inputFilesLV,           SIGNAL(currentRowChanged(int)), this, SLOT(inputFilesLVChanged()));
	connect(ui.addBtn,				   SIGNAL(clicked()), this, SLOT(onAddBtnClick()));
	connect(ui.btnAppend,			   SIGNAL(clicked()), this, SLOT(onAppendButtonClick()));
	connect(ui.removeFile,			   SIGNAL(clicked()), this, SLOT(onRemoveBtnClick()));
	connect(ui.removeTrackBtn,		   SIGNAL(clicked()), this, SLOT(onRemoveTrackButtonClick()));
	connect(ui.moveupBtn,			   SIGNAL(clicked()), this, SLOT(onMoveUpButtonCLick()));
	connect(ui.movedownBtn,			   SIGNAL(clicked()), this, SLOT(onMoveDownButtonCLick()));
	connect(ui.checkFPS,			   SIGNAL(stateChanged(int)), this, SLOT(onVideoCheckBoxChanged(int)));
	connect(ui.checkBoxLevel,		   SIGNAL(stateChanged(int)), this, SLOT(onVideoCheckBoxChanged(int)));
	connect(ui.checkBoxSEI,            SIGNAL(stateChanged(int)), this, SLOT(onVideoCheckBoxChanged(int)));
	connect(ui.checkBoxSPS,            SIGNAL(stateChanged(int)), this, SLOT(onVideoCheckBoxChanged(int)));
	connect(ui.checkBoxRemovePulldown, SIGNAL(stateChanged(int)), this, SLOT(onPulldownCheckBoxChanged(int)));
	connect(ui.comboBoxFPS,            SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoComboBoxChanged(int)));
	connect(ui.comboBoxLevel,          SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoComboBoxChanged(int)));
	connect(ui.comboBoxAR,             SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoComboBoxChanged(int)));
	connect(ui.checkBoxKeepFps,        SIGNAL(stateChanged(int)), this, SLOT(onAudioSubtitlesCheckBoxChanged(int)));
	connect(ui.dtsDwnConvert,          SIGNAL(stateChanged(int)), this, SLOT(onAudioSubtitlesCheckBoxChanged(int)));
	connect(ui.langComboBox,           SIGNAL(currentIndexChanged(int)), this, SLOT(onAudioSubtitlesComboBoxChanged(int)));
	connect(ui.editDelay,			   SIGNAL(valueChanged (int)), this, SLOT(onEditDelayChanged(int)));
	connect(ui.fontButton,			   SIGNAL(clicked()), this, SLOT(onFontBtnClicked()));
	connect(ui.colorButton,			   SIGNAL(clicked()), this, SLOT(onColorBtnClicked()));
	connect(ui.checkBoxVBR,			   SIGNAL(clicked()), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.spinBoxMplsNum,		   SIGNAL(valueChanged(int)), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.spinBoxM2tsNum,		   SIGNAL(valueChanged(int)), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxBlankPL,		   SIGNAL(clicked()), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.BlackplaylistCombo,	   SIGNAL(valueChanged(int)), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxNewAudioPes,	   SIGNAL(clicked()), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxCrop,           SIGNAL(stateChanged(int)), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxRVBR,		   SIGNAL(clicked()), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxCBR,			   SIGNAL(clicked()), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.checkBoxuseAsynIO,	   SIGNAL(stateChanged(int)), this, SLOT(onGeneralCheckboxClicked()));
	connect(ui.editVBVLen,			   SIGNAL(valueChanged(int)), this, SLOT(onGeneralSpinboxValueChanged()));
	connect(ui.editMaxBitrate,		   SIGNAL(valueChanged(double)), this, SLOT(onGeneralSpinboxValueChanged()));
	connect(ui.editMinBitrate,		   SIGNAL(valueChanged(double)), this, SLOT(onGeneralSpinboxValueChanged()));
	connect(ui.editCBRBitrate,		   SIGNAL(valueChanged(double)), this, SLOT(onGeneralSpinboxValueChanged()));
	connect(ui.radioButtonAutoChapter,	  SIGNAL(clicked()), this, SLOT(onChapterParamsChanged()));
	connect(ui.radioButtonNoChapters,	  SIGNAL(clicked()), this, SLOT(onChapterParamsChanged()));
	connect(ui.radioButtonCustomChapters, SIGNAL(clicked()), this, SLOT(onChapterParamsChanged()));
	connect(ui.spinEditChapterLen,		  SIGNAL(valueChanged(int)), this, SLOT(onChapterParamsChanged()));
	connect(ui.memoChapters,		      SIGNAL(textChanged()), this, SLOT(onChapterParamsChanged()));
	connect(ui.noSplit,					  SIGNAL(clicked()), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.splitByDuration,		      SIGNAL(clicked()), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.splitBySize,				  SIGNAL(clicked()), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.spinEditSplitDuration,	  SIGNAL(valueChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.editSplitSize,		      SIGNAL(valueChanged(double)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.comboBoxMeasure,		      SIGNAL(currentIndexChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.checkBoxCut,				  SIGNAL(stateChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.editCutStart,		      SIGNAL(valueChanged(double)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.editCutEnd,				  SIGNAL(valueChanged(double)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.measureCutStart,		      SIGNAL(currentIndexChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.measureCutEnd,		      SIGNAL(currentIndexChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.spinEditBorder,		      SIGNAL(valueChanged(int)), this, SLOT(onSplitCutParamsChanged()));
	connect(ui.spinEditOffset,		      SIGNAL(valueChanged(int)), this, SLOT(onFontParamsChanged()));
	connect(ui.radioButtonTS,		      SIGNAL(clicked()), this, SLOT(RadioButtonMuxClick()));
	connect(ui.radioButtonM2TS,		      SIGNAL(clicked()), this, SLOT(RadioButtonMuxClick()));
	connect(ui.radioButtonBluRay,		  SIGNAL(clicked()), this, SLOT(RadioButtonMuxClick()));
	connect(ui.radioButtonAVCHD,		  SIGNAL(clicked()), this, SLOT(RadioButtonMuxClick()));
	connect(ui.radioButtonDemux,		  SIGNAL(clicked()), this, SLOT(RadioButtonMuxClick()));
	connect(ui.outFileName,				  SIGNAL(textChanged(const QString&)), this, SLOT(outFileNameChanged()));
	connect(ui.btnBrowse,				  SIGNAL(clicked()), this, SLOT(saveFileDialog()));
	connect(ui.buttonMux,				  SIGNAL(clicked()), this, SLOT(startMuxing()));
	connect(ui.buttonSaveMeta,			  SIGNAL(clicked()), this, SLOT(saveMetaFileBtnClick()));
	connect( &proc, SIGNAL(readyReadStandardOutput()), this, SLOT(readFromStdout()) );
	connect( &proc, SIGNAL(readyReadStandardError()), this, SLOT(readFromStderr()) );
	connect( &proc, SIGNAL(finished(int, QProcess::ExitStatus )), this, SLOT(onProcessFinished(int, QProcess::ExitStatus )) );
	connect( &proc, SIGNAL(error( QProcess::ProcessError)), this, SLOT(onProcessError(QProcess::ProcessError)) );

	lastSelectedDir = QCoreApplication::applicationDirPath();
	ui.langComboBox->addItem("und (Undetermined)");
	ui.langComboBox->addItem("--------- common ---------");
	for (int i = 0; i < sizeof(shortLangList)/2/sizeof(char*); ++i) {
		const char* addr = shortLangList[i][0];
		ui.langComboBox->addItem(QString(shortLangList[i][0]) + " (" + shortLangList[i][1] + ")", (qlonglong) (void*) addr);
	}
	ui.langComboBox->addItem("---------- all ----------");
	for (int i = 0; i < sizeof(fullLangList)/2/sizeof(char*); ++i) {
		const char* addr = fullLangList[i][0];
		ui.langComboBox->addItem(QString(fullLangList[i][0]) + " (" + fullLangList[i][1] + ")", (qlonglong) (void*) addr);
	}
	trackLVItemSelectionChanged();
}

TsMuxerWindow::~TsMuxerWindow()
{
	delete sound;
	delete tempSoundFile;
}

void TsMuxerWindow::getCodecInfo()
{
	codecList.clear();
	int p;
	QtvCodecInfo* codecInfo = 0;
	int lastTrackID = 0;
	QString tmpStr;
	bool firstMark = true;
	QList<QString> stringList;
	codecList.clear();
	mplsFileList.clear();
	for (int i = 0; i < procStdOutput.size(); ++i)
	{
		p = procStdOutput[i].indexOf("Track ID:    ");
		if (p >= 0) {
			lastTrackID = procStdOutput[i].mid(QString("Track ID:    ").length()).toInt();
			codecList << QtvCodecInfo();
			codecInfo = &(codecList.back());
			codecInfo->trackID = lastTrackID;
		}

		p = procStdOutput[i].indexOf("Stream type: ");
		if (p >= 0) {
			if (lastTrackID == 0) {
				codecList << QtvCodecInfo();
				codecInfo = &(codecList.back());
			}
			codecInfo->descr = "Can't detect codec";
			codecInfo->displayName = procStdOutput[i].mid(QString("Stream type: ").length());
			if (codecInfo->displayName != "H.264") {
				codecInfo->addSEI = false;
				codecInfo->addSPS = false;
			}
			lastTrackID = 0;
		}
		p = procStdOutput[i].indexOf("Stream ID:   ");
		if (p >= 0) 
			codecInfo->programName = procStdOutput[i].mid(QString("Stream ID:   ").length());
		p = procStdOutput[i].indexOf("Stream info: ");
		if (p >= 0) 
		  codecInfo->descr = procStdOutput[i].mid(QString("Stream info: ").length());
		p = procStdOutput[i].indexOf("Stream lang: ");
		if (p >= 0) 
		  codecInfo->lang = procStdOutput[i].mid(QString("Stream lang: ").length());

		p = procStdOutput[i].indexOf("Stream delay: ");
		if (p >= 0) {
			tmpStr = procStdOutput[i].mid(QString("Stream delay: ").length());
			codecInfo->delay = tmpStr.toInt();
		}
		p = procStdOutput[i].indexOf("Error: ");
		if (p == 0) {
			tmpStr = procStdOutput[i].mid(QString("Error: ").length());
			QMessageBox msgBox(this);
			msgBox. setWindowTitle("Not supported");
			msgBox.setText(tmpStr);
			msgBox.setIcon(QMessageBox::Warning);
			msgBox.setStandardButtons(QMessageBox::Ok);
			msgBox.exec();
		}

		p = procStdOutput[i].indexOf("File #");
		if (p == 0) {
			tmpStr = procStdOutput[i].mid(17); // todo: check here
			mplsFileList << tmpStr;
		}
		p = procStdOutput[i].indexOf("Marks: ");
		if (p == 0) {
			if (firstMark) {
				firstMark = false;
				ui.memoChapters->clear();
				ui.radioButtonCustomChapters->setChecked(true);
			}
			stringList = procStdOutput[i].mid(7).split(' ');
			for (int k = 0; k < stringList.size(); ++k)
				if (!stringList[k].isEmpty())
					ui.memoChapters->insertPlainText(stringList[k] + QString('\n'));
		}
	}
	if (codecList.isEmpty()) {
		QMessageBox msgBox(this);
		msgBox.setWindowTitle("Unsupported format");
		msgBox.setText(QString("Can't detect stream type. File name: \"") + newFileName + "\"");
		msgBox.setIcon(QMessageBox::Warning);
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.exec();
		return;
	}
	emit codecListReady();
}

int TsMuxerWindow::findLangByCode(const QString& code)
{
	for (int i = 0; i < ui.langComboBox->count(); ++i) {
		char* addr = (char*) ui.langComboBox->itemData(i).toLongLong();
		if (addr && code == addr) 
			return i;
	}
	return 0;
}

QtvCodecInfo* TsMuxerWindow::getCurrentCodec()
{
	if (ui.trackLV->currentRow() == -1)
		return 0;
	long iCodec = ui.trackLV->item(ui.trackLV->currentRow(), 0)->data(Qt::UserRole).toLongLong();
	return iCodec ? (QtvCodecInfo*) (void*) iCodec : 0;
}

void TsMuxerWindow::onVideoComboBoxChanged(int index)
{
	Q_UNUSED(index);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	codecInfo->fpsText = ui.comboBoxFPS->itemText(ui.comboBoxFPS->currentIndex());
	codecInfo->levelText = ui.comboBoxLevel->itemText(ui.comboBoxLevel->currentIndex());
	codecInfo->arText = ui.comboBoxAR->itemText(ui.comboBoxAR->currentIndex());
	updateMetaLines();
}

void TsMuxerWindow::onVideoCheckBoxChanged(int state)
{
	Q_UNUSED(state);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	ui.comboBoxFPS->setEnabled(ui.checkFPS->isChecked());
	codecInfo->checkFPS = ui.checkFPS->isChecked();
	ui.comboBoxLevel->setEnabled(ui.checkBoxLevel->isChecked());
	codecInfo->checkLevel = ui.checkBoxLevel->isChecked();
	codecInfo->addSEI = ui.checkBoxSEI->isChecked();
	codecInfo->addSPS = ui.checkBoxSPS->isChecked();
	updateMetaLines();
}

void TsMuxerWindow::onAudioSubtitlesCheckBoxChanged(int state)
{
	Q_UNUSED(state);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	codecInfo->bindFps = ui.checkBoxKeepFps->isChecked();
	codecInfo->dtsDownconvert = ui.dtsDwnConvert->isChecked();
	updateMetaLines();
}

void TsMuxerWindow::onAudioSubtitlesComboBoxChanged(int index)
{
	Q_UNUSED(index);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;

	char* addr = (char*) ui.langComboBox->itemData(ui.langComboBox->currentIndex()).toLongLong();
	if (addr) 
		codecInfo->lang = addr;
	else
		codecInfo->lang.clear();
	ui.trackLV->item(ui.trackLV->currentRow(),3)->setText(codecInfo->lang);
	updateMetaLines();
}

void TsMuxerWindow::onEditDelayChanged(int i)
{
	Q_UNUSED(i);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	codecInfo->delay = ui.editDelay->value();
	updateMetaLines();
}

void TsMuxerWindow::onPulldownCheckBoxChanged(int state)
{
	Q_UNUSED(state);
	if (trackLVIngoreOnChange)
		return;
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	if (ui.checkBoxRemovePulldown->isEnabled())
	{
		if (ui.checkBoxRemovePulldown->isChecked()) {
			codecInfo->delPulldown = 1;
			if (codecInfo->fpsTextOrig == "29.97") 
			{
				ui.checkFPS->setChecked(true);
				ui.checkFPS->setEnabled(true);
				codecInfo->checkFPS = true;
				ui.comboBoxFPS->setEnabled(true);
				ui.comboBoxFPS->setCurrentIndex(3);
				codecInfo->fpsText = "24000/1001";
				setComboBoxText(ui.comboBoxFPS, "24000/1001");
			}
		}
		else
			codecInfo->delPulldown = 0;
	}
	else
		codecInfo->delPulldown = -1;
	updateMetaLines();
}

void TsMuxerWindow::onAddBtnClick()
{
	QString fileName = QDir::toNativeSeparators(QFileDialog::getOpenFileName(this, tr("Add media file"), lastSelectedDir, tr(fileDialogFilter)));
	if (fileName.isEmpty())
		return;
	lastSelectedDir = fileName;
	//disconnect();
	addFileList.clear();
	addFileList << QUrl::fromLocalFile(fileName);
	addFile();
}

void TsMuxerWindow::addFile()
{
	if (addFileList.isEmpty())
		return;
	newFileName = QDir::toNativeSeparators(addFileList[0].toLocalFile());
	addFileList.removeAt(0);
	if (!checkFileDuplicate(newFileName)) 
		return;
	//disconnect(this, SIGNAL(tsMuxerSuccessFinished()));
	//disconnect(this, SIGNAL(codecListReady()));
	disconnect();
	connect(this, SIGNAL(tsMuxerSuccessFinished()), this, SLOT(getCodecInfo()));
	connect(this, SIGNAL(codecListReady()), this, SLOT(continueAddFile()));
	connect(this, SIGNAL(fileAdded()), this, SLOT(addFile()));
	runInMuxMode = false;
	shellExecute(QDir::toNativeSeparators(QCoreApplication::applicationDirPath()) + QDir::separator() + "tsMuxeR", QStringList() << newFileName);
}

bool TsMuxerWindow::checkFileDuplicate(const QString& fileName)
{
	QString t = myUnquoteStr(fileName).trimmed();
	for (int i = 0; i < ui.inputFilesLV->count(); ++i)
		if (myUnquoteStr(ui.inputFilesLV->item(i)->text()) == t) 
		{
			QMessageBox msgBox(this);
			msgBox. setWindowTitle("File already exist");
			msgBox.setText("File \"" + fileName + "\" already exist");
			msgBox.setIcon(QMessageBox::Warning);
			msgBox.setStandardButtons(QMessageBox::Ok);
			msgBox.exec();
			return false;
		}
	return true;
}

void TsMuxerWindow::setComboBoxText(QComboBox* comboBox, const QString& text)
{
	for (int k = 0; k < comboBox->count(); ++k)
	{
		if (comboBox->itemText(k) == text) {
			comboBox->setCurrentIndex(k);
			return;
		}
	}
	comboBox->addItem(text);
	comboBox->setCurrentIndex(comboBox->count()-1);
}

void TsMuxerWindow::trackLVItemSelectionChanged()
{
	if (trackLVIngoreOnChange)
		return;
 	while(ui.tabWidgetTracks->count())
		ui.tabWidgetTracks->removeTab(0);
	if (ui.trackLV->currentRow() == -1)
	{
		ui.tabWidgetTracks->addTab(ui.tabSheetFake, TI_DEFAULT_TAB_NAME);
		return;
	}
	QtvCodecInfo* codecInfo = getCurrentCodec();
	if(!codecInfo)
		return;
	trackLVIngoreOnChange = true;
	if (ui.trackLV->rowCount() >= 1)
	{
		if (codecInfo->displayName == "H.264" || codecInfo->displayName == "VC-1" || codecInfo->displayName == "MPEG-2")
		{
			ui.tabWidgetTracks->addTab(ui.tabSheetVideo, TI_DEFAULT_TAB_NAME);

			ui.checkFPS->setChecked(codecInfo->checkFPS);
			ui.checkBoxLevel->setChecked(codecInfo->checkLevel);
			ui.comboBoxFPS->setEnabled(ui.checkFPS->isChecked());
			ui.comboBoxLevel->setEnabled(ui.checkBoxLevel->isChecked());
			ui.checkBoxSEI->setChecked(codecInfo->addSEI);
			ui.checkBoxSPS->setChecked(codecInfo->addSPS);
			ui.checkBoxRemovePulldown->setChecked(codecInfo->delPulldown == 1);
			ui.checkBoxRemovePulldown->setEnabled(codecInfo->delPulldown >=0 );

			setComboBoxText(ui.comboBoxFPS, codecInfo->fpsText);
			if (!codecInfo->arText.isEmpty()) 
				setComboBoxText(ui.comboBoxAR, codecInfo->arText);
			else
				ui.comboBoxAR->setCurrentIndex(0);
			ui.checkBoxLevel->setEnabled(codecInfo->displayName == "H.264");
			if (ui.checkBoxLevel->isEnabled())
				setComboBoxText(ui.comboBoxLevel, codecInfo->levelText);
			else
				ui.comboBoxLevel->setCurrentIndex(0);
			ui.checkBoxSEI->setEnabled(ui.checkBoxLevel->isEnabled());
			ui.checkBoxSPS->setEnabled(ui.checkBoxLevel->isEnabled());
			ui.comboBoxAR->setEnabled(codecInfo->displayName == "MPEG-2");
			ui.labelAR->setEnabled(ui.comboBoxAR->isEnabled());
		}
		else
		{			
			ui.tabWidgetTracks->addTab(ui.tabSheetAudio, TI_DEFAULT_TAB_NAME);
			if (codecInfo->displayName == "LPCM") 
				ui.tabWidgetTracks->addTab(ui.demuxLpcmOptions, TI_DEMUX_TAB_NAME);
				
			if (codecInfo->displayName == "DTS-HD")
				ui.dtsDwnConvert->setText("Downconvert DTS-HD to DTS");
			else if (codecInfo->displayName == "TRUE-HD")
				ui.dtsDwnConvert->setText("Downconvert TRUE-HD to AC3");
			else
				ui.dtsDwnConvert->setText("Downconvert HD audio");
			ui.dtsDwnConvert->setEnabled((codecInfo->displayName == "DTS-HD"  || codecInfo->displayName == "TRUE-HD"));

			ui.langComboBox->setCurrentIndex(findLangByCode(codecInfo->lang));
			ui.dtsDwnConvert->setVisible(codecInfo->displayName != "PGS" && codecInfo->displayName != "SRT");
			ui.checkBoxKeepFps->setVisible(codecInfo->displayName == "PGS"); //not dtsDwnConvert.visible;
			ui.imageSubtitles->setVisible(!ui.dtsDwnConvert->isVisible());
			ui.imageAudio->setVisible(ui.dtsDwnConvert->isVisible());

			ui.editDelay->setValue(codecInfo->delay);
			ui.dtsDwnConvert->setChecked(codecInfo->dtsDownconvert);
			ui.checkBoxKeepFps->setChecked(codecInfo->bindFps);
			ui.editDelay->setEnabled(!ui.radioButtonDemux->isChecked());
		}
	}
	trackLVIngoreOnChange = false;
	trackLVItemChanged(0);
}

void TsMuxerWindow::trackLVItemChanged(QTableWidgetItem* item)
{
	Q_UNUSED(item);
    updateMetaLines();
	ui.moveupBtn->setEnabled(ui.trackLV->currentItem() != 0);
    ui.movedownBtn->setEnabled(ui.trackLV->currentItem() != 0);
    ui.removeTrackBtn->setEnabled(ui.trackLV->currentItem() != 0);
	if (ui.trackLV->rowCount() == 0)
		oldFileName.clear();
}

void TsMuxerWindow::inputFilesLVChanged()
{
	QListWidgetItem* itm = ui.inputFilesLV->currentItem();
	if (!itm) {
		ui.btnAppend->setEnabled(false);
		ui.removeFile->setEnabled(false);
		return;
	}
	ui.btnAppend->setEnabled(itm->data(Qt::UserRole).toInt() !=  1 && ui.buttonMux->isEnabled());
	ui.removeFile->setEnabled(itm->data(Qt::UserRole).toInt() !=  1 || itm->text().left(4) != " ++ ");
}

void TsMuxerWindow::modifyOutFileName(const QString fileName)
{
	QFileInfo fi(unquoteStr(fileName));
	QString name = fi.fileName().mid(0, fi.fileName().length() - fi.suffix().length() - 1);
	QString path = QDir::toNativeSeparators(fi.absolutePath());
	if (ui.radioButtonTS->isChecked())
		ui.outFileName->setText(path + QDir::separator() + name + ".ts");
	else if (ui.radioButtonM2TS->isChecked())
		ui.outFileName->setText(path + QDir::separator() + name + ".m2ts");
	else
		ui.outFileName->setText(path);
	if (ui.outFileName->text() == fileName) 
		ui.outFileName->setText(path + QDir::separator() + name + "_new." + fi.suffix());
}

void TsMuxerWindow::continueAddFile()
{
	double fps;
	double level;
	trackLVIngoreOnChange = true;
	bool firstWarn = true;
	int firstAddedIndex = -1;
	for (int i = 0; i < codecList.size(); ++i)
	{
		QtvCodecInfo info = codecList[i];
		if (info.displayName.isEmpty())
		{
			QMessageBox msgBox(this);
			msgBox.setWindowTitle("Unsupported format");
			msgBox.setIcon(QMessageBox::Warning);
			msgBox.setStandardButtons(QMessageBox::Ok);
			if (codecList.size() == 0) {
				msgBox.setText(QString("Unsupported format or all tracks are not recognized. File name: \"") + newFileName + "\"");
				msgBox.exec();
				trackLVIngoreOnChange = false;
				return;
			}
			else {
				if (firstWarn) {
					msgBox.setText(QString("Some tracks not recognized. This tracks was ignored. File name: \"") + newFileName + "\"");
					msgBox.exec();
					firstWarn = false;
				}
				continue;
			}
		}
		if (mplsFileList.isEmpty()) 
			info.fileList << newFileName;
		else {
			info.fileList << mplsFileList[0];
			QFileInfo fileInfo(unquoteStr(newFileName));
			info.mplsFile = fileInfo.baseName();
		}
		if (info.descr.indexOf("not found") >= 0) {
			fps = 23.976;
			info.checkFPS = true;
		}
		else
			fps = extractFloatFromDescr(info.descr, "Frame rate: ");
		info.width  = extractFloatFromDescr(info.descr, "Resolution: ") + 0.5;
		info.height = extractFloatFromDescr(info.descr, QString::number(info.width) + ":") + 0.5;
		if (qAbs(fps - 23.976) < EPS)
			info.fpsText = "24000/1001";
		else if (qAbs(fps - 29.97) < EPS)
			info.fpsText = "30000/1001";
		else
			info.fpsText = myFloatToStr(fps);
		info.fpsTextOrig = myFloatToStr(fps);
		level = extractFloatFromDescr(info.descr, "@");
		info.levelText = myFloatToStr(level);
		if (info.descr.indexOf("pulldown") >= 0)
			info.delPulldown = 0;
		ui.trackLV->setRowCount(ui.trackLV->rowCount()+1);
		ui.trackLV->setRowHeight(ui.trackLV->rowCount()-1, 18);
		QTableWidgetItem *item = new QTableWidgetItem("");
		item->setCheckState(Qt::Checked);
		QtvCodecInfo* __info = new QtvCodecInfo();
		*__info = info;
		item->setData(Qt::UserRole, (qlonglong) (void*) __info);
		ui.trackLV->setCurrentItem(item);

		ui.trackLV->setItem(ui.trackLV->rowCount()-1, 0, item);
		item = new QTableWidgetItem(newFileName);
		item ->setFlags(item ->flags() & (~Qt::ItemIsEditable));
		ui.trackLV->setItem(ui.trackLV->rowCount()-1, 1, item);
		item = new QTableWidgetItem(info.displayName);
		item ->setFlags(item ->flags() & (~Qt::ItemIsEditable));
		ui.trackLV->setItem(ui.trackLV->rowCount()-1, 2, item);
		item = new QTableWidgetItem(info.lang);
		item ->setFlags(item ->flags() & (~Qt::ItemIsEditable));
		ui.trackLV->setItem(ui.trackLV->rowCount()-1, 3, item);
		item = new QTableWidgetItem(info.descr);
		item ->setFlags(item ->flags() & (~Qt::ItemIsEditable));
		ui.trackLV->setItem(ui.trackLV->rowCount()-1, 4, item);
		if (firstAddedIndex == -1)
			firstAddedIndex = ui.trackLV->rowCount()-1;
	}
	if (firstAddedIndex >= 0) {
		ui.trackLV->setRangeSelected(QTableWidgetSelectionRange(firstAddedIndex, 0, ui.trackLV->rowCount()-1, 4), true);
		ui.trackLV->setCurrentCell(firstAddedIndex, 0);
	}
	ui.inputFilesLV->addItem(newFileName);
	if (!mplsFileList.empty())
		ui.inputFilesLV->item(ui.inputFilesLV->count()-1)->setData(Qt::UserRole, 1);
	int index = ui.inputFilesLV->count()-1;
	ui.inputFilesLV->setCurrentItem(ui.inputFilesLV->item(index));
	if (mplsFileList.size() > 0)
		doAppendInt(mplsFileList[0], newFileName);
	for (int mplsCnt = 1; mplsCnt < mplsFileList.size(); ++mplsCnt)
		doAppendInt(mplsFileList[mplsCnt], mplsFileList[mplsCnt-1]);
	ui.inputFilesLV->setCurrentItem(ui.inputFilesLV->item(index));
	updateMetaLines();
	ui.moveupBtn->setEnabled(ui.trackLV->currentRow() >= 0);
	ui.movedownBtn->setEnabled(ui.trackLV->currentRow() >= 0);
	ui.removeTrackBtn->setEnabled(ui.trackLV->currentRow() >= 0);
	if (!outFileNameModified)
	{
		modifyOutFileName(newFileName);
		outFileNameModified = true;
	}
	trackLVIngoreOnChange = false;	
	trackLVItemSelectionChanged();
	emit fileAdded();
}

void splitLines(const QString& str, QList<QString>& strList)
{
	strList = str.split('\n');
	for (int i = 0; i < strList.size(); ++i)
	{
		if (!strList[i].isEmpty() && strList[i].at(0) == '\r')
			strList[i] = strList[i].mid(1);
		else if (!strList[i].isEmpty() && strList[i].at(strList[i].length()-1) == '\r')
			strList[i] = strList[i].mid(0, strList[i].length()-1);
	}
}

void TsMuxerWindow::addLines(const QByteArray& arr, QList<QString>& outList, bool isError)
{
#ifdef WIN32
	OemToAnsi(arr.data(), (char*) arr.data());
#endif
	QString str = QString::fromLocal8Bit(arr);
	str.fromLocal8Bit(arr);
	QList<QString> strList;
	splitLines(str, strList);
	QString text;
	for (int i = 0; i < strList.size(); ++i)
	{
		if (strList[i].trimmed().isEmpty())
			continue;
		int p = strList[i].indexOf("% complete");
		if (p >= 0) 
		{
			int numStartPos = 0;
			for (int j = 0; j < strList[i].length(); ++j) {
				if (strList[i].at(j) >= '0' && strList[i].at(j) <= '9') {
					numStartPos = j;
					break;
				}
			}
			QString progress = strList[i].mid(numStartPos, p - numStartPos);
			float tmpVal = progress.toFloat();
			if (tmpVal != 0 && runInMuxMode)
				muxForm.setProgress(tmpVal*10 + 0.5); // todo: uncomment here
		}
		else {
			if (runInMuxMode)
			{
				if (!text.isEmpty())
					text += '\n';
				text += strList[i];
			}
			else			
				outList << strList[i];
		}
	}
	if (runInMuxMode && !text.isEmpty())
	{
		if (isError)
			muxForm.addStdErrLine(text);
		else
			muxForm.addStdOutLine(text);
	}
}

void TsMuxerWindow::readFromStdout()
{
	addLines(proc.readAllStandardOutput(), procStdOutput, false);
}

void TsMuxerWindow::readFromStderr()
{
	QByteArray data(proc.readAllStandardError());
	addLines(data, procErrOutput, true);
}

void TsMuxerWindow::myPlaySound(const QString& fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::ReadOnly))
		return;
	qint64 fSize = file.size();
	char* data  = new char[fSize];
	qint64 readed = file.read(data, fSize);
	QFileInfo fi(fileName);
	QString outFileName = QDir::toNativeSeparators(QDir::tempPath()) + QDir::separator()  + "tsMuxeR_" + fi.fileName();
	delete sound;
	delete tempSoundFile;
	tempSoundFile = new QTemporaryFile(outFileName);
	
	if (!tempSoundFile->open())
	{
		delete [] data;
		delete tempSoundFile;
		tempSoundFile = 0;
		return;
	}
	
	tempSoundFile->write(data, readed);
	QString tmpFileName = tempSoundFile->fileName();
	tempSoundFile->close();
	sound = new QSound(tmpFileName);
	sound->play();
	file.close();
	delete [] data;
}

void TsMuxerWindow::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
	processFinished = true;
	if (!metaName.isEmpty()) 
	{
		QFile::remove(metaName);
		metaName.clear();
		if (ui.checkBoxSound->isChecked())
		{
			if (exitCode == 0)
				myPlaySound(":/sounds/success.wav");
			else
				myPlaySound(":/sounds/fail.wav");
		}
	}
	processExitCode = exitCode;
	processExitStatus = exitStatus;
	muxForm.muxFinished(processExitCode, ui.radioButtonDemux->isChecked() ? "Demux" : "Mux");
	ui.buttonMux->setEnabled(true);
	ui.addBtn->setEnabled(true);
	inputFilesLVChanged();
	if (processExitCode == 0)
		emit tsMuxerSuccessFinished();
}

void TsMuxerWindow::onProcessError(QProcess::ProcessError error)
{
	processExitCode = -1;
	if (!metaName.isEmpty()) 
	{
		QFile::remove(metaName);
		metaName.clear();
	}
	processFinished = true;
	QMessageBox msgBox(this);
	QString text;
	msgBox. setWindowTitle("tsMuxeR error");
	switch(error) {
		case QProcess::FailedToStart:
			msgBox.setText("tsMuxeR not found!");
			ui.buttonMux->setEnabled(true);
			ui.addBtn->setEnabled(true);
			inputFilesLVChanged();
			break;
		case QProcess::Crashed:
			// process killed
			if (runInMuxMode)
				return;
			for (int i = 0; i < procErrOutput.size(); ++i) {
				if (i > 0)
					text += '\n';
				text += procErrOutput[i];
			}
			msgBox.setText(text);
			break;
		default:
			msgBox.setText("Can't execute tsMuxeR!");
			break;
	}
	msgBox.setIcon(QMessageBox::Critical);
	msgBox.setStandardButtons(QMessageBox::Ok);
	msgBox.exec();
}

void TsMuxerWindow::shellExecute(const QString& process, const QStringList& args)
{
	ui.buttonMux->setEnabled(false);
	procStdOutput.clear();
	procErrOutput.clear();
	processFinished = false;
	processExitCode = -1;
	proc.start(process, args);
	if (muxForm.isVisible())
		muxForm.setProcess(&proc);
}

void TsMuxerWindow::doAppendInt(const QString& fileName, const QString& parentFileName)
{
	QListWidgetItem* item = new QListWidgetItem(QString(" ++ ") + fileName);
	ui.inputFilesLV->insertItem(ui.inputFilesLV->currentRow() + 1, item);
    if (mplsFileList.size() > 0)
		item->setData(Qt::UserRole, 1);
	ui.inputFilesLV->setCurrentItem(item);

	for (int i = 0; i < ui.trackLV->rowCount(); ++i)
	{
		long data = ui.trackLV->item(i,0)->data(Qt::UserRole).toLongLong();
		if (!data)
			continue;
		QtvCodecInfo* info = (QtvCodecInfo*) (void*) data;
		for (int j = 0; j < info->fileList.size(); ++j) 
		{
			if (info->fileList[j] == parentFileName) {
				info->fileList.insert(j+1, fileName);
				break;
			}
		}
	}
}

bool TsMuxerWindow::isVideoCropped()
{
	for (int i = 0; i < ui.trackLV->rowCount(); ++i) 
	{
		long iCodec = ui.trackLV->item(i,0)->data(Qt::UserRole).toLongLong();
		if (!iCodec) continue;
		QtvCodecInfo* codecInfo = (QtvCodecInfo*) (void*) iCodec;
		if (codecInfo->displayName == "H.264" || codecInfo->displayName == "VC-1" || codecInfo->displayName == "MPEG-2") 
		{
			if (codecInfo->height < 1080 && codecInfo->height != 720 && 
				codecInfo->height != 576 && codecInfo->height != 480)
				return true;
		}
	}
	return false;
}

QString TsMuxerWindow::getMuxOpts()
{
	QString rez =  "MUXOPT --no-pcr-on-video-pid ";
	if (ui.checkBoxNewAudioPes->isChecked())
		rez += "--new-audio-pes ";
	if (ui.radioButtonBluRay->isChecked())
		rez += "--blu-ray ";
	else if (ui.radioButtonAVCHD->isChecked())
		rez += "--avchd ";
	else if (ui.radioButtonDemux->isChecked())
		rez += "--demux ";
	if (ui.checkBoxCBR->isChecked())
		rez += "--cbr --bitrate=" + toFixedDecPoint(QString::number(ui.editCBRBitrate->value(),'f',3));
	else {
		rez += "--vbr ";
		if (ui.checkBoxRVBR->isChecked()) {
			rez += QString("--minbitrate=")  + toFixedDecPoint(QString::number(ui.editMinBitrate->value(),'f',3));
			rez += QString(" --maxbitrate=") + toFixedDecPoint(QString::number(ui.editMaxBitrate->value(),'f',3));
		}
	}
	if (!ui.checkBoxuseAsynIO->isChecked())
		rez += " --no-asyncio";
	if (ui.radioButtonAVCHD->isChecked() || ui.radioButtonBluRay->isChecked())
	{
		if (ui.checkBoxBlankPL->isChecked() && isVideoCropped()) {
			rez += " --insertBlankPL";
			if (ui.BlackplaylistCombo->value() != 1900)
				rez += QString(" --blankOffset=") + QString::number(ui.BlackplaylistCombo->value());
		}
		if (ui.spinBoxMplsNum->value() > 0)
			rez += " --mplsOffset=" + QString::number(ui.spinBoxMplsNum->value());
		if (ui.spinBoxM2tsNum->value() > 0)
			rez += " --m2tsOffset=" + QString::number(ui.spinBoxM2tsNum->value());
	}
	if (ui.radioButtonBluRay->isChecked() || ui.radioButtonAVCHD->isChecked()) 
	{
		if (ui.radioButtonAutoChapter->isChecked()) 
			rez += " --auto-chapters=" + QString::number(ui.spinEditChapterLen->value());
		if (ui.radioButtonCustomChapters->isChecked()) {
			QString custChapStr;
			QList<QString> lines;
			splitLines(ui.memoChapters->toPlainText(), lines);
			for (int i = 0; i < lines.size(); ++i) 
			{
				QString tmpStr = lines[i].trimmed();
				if (!tmpStr.isEmpty()) {
					if (!custChapStr.isEmpty()) 
						custChapStr += ';';
					custChapStr += tmpStr;
				}
			}
			rez += QString(" --custom-chapters=") + custChapStr;
		}
	}
	if (ui.splitByDuration->isChecked())
		rez += QString(" --split-duration=") + ui.spinEditSplitDuration->text();
	if (ui.splitBySize->isChecked()) 
		rez += QString(" --split-size=") + myFloatToStr(myStrToFloat(ui.editSplitSize->text())) + ui.comboBoxMeasure->currentText();
	double startCut = myStrToFloat(ui.editCutStart->text());
	double endCut = myStrToFloat(ui.editCutEnd->text());
	if (startCut > 0 && ui.checkBoxCut->isChecked()) 
		rez += QString(" --cut-start=") + myFloatToStr(startCut) + ui.measureCutStart->currentText();
	if (endCut > 0  && ui.checkBoxCut->isChecked())
		rez += QString(" --cut-end=") + myFloatToStr(endCut) + ui.measureCutEnd->currentText();
	int vbvLen = ui.editVBVLen->value();
	if (vbvLen > 0) 
		rez += " --vbv-len=" + QString::number(vbvLen);
	return rez;
}

int getCharsetCode(const QString& name)
{
	Q_UNUSED(name);
	return 0; // todo: refactor this function
}

QString TsMuxerWindow::getSrtParams()
{
	QString rez;
	if (ui.listViewFont->rowCount() < 5) 
		return rez;
	rez = QString(",font-name=\"") + ui.listViewFont->item(0,1)->text();
	rez += QString("\",font-size=") + ui.listViewFont->item(1,1)->text();
	rez += QString(",font-color=") + ui.listViewFont->item(2,1)->text();
	int charsetCode = getCharsetCode(ui.listViewFont->item(3,1)->text());
	if (charsetCode)
		rez += QString(",font-charset=") + QString::number(charsetCode);
	if (ui.listViewFont->item(4,1)->text().indexOf("Italic") >= 0) 
		rez += ",font-italic";
	if (ui.listViewFont->item(4,1)->text().indexOf("Bold") >= 0) 
		rez += ",font-bold";
	if (ui.listViewFont->item(4,1)->text().indexOf("Underline") >= 0) 
		rez += ",font-underline";
	if (ui.listViewFont->item(4,1)->text().indexOf("Strikeout") >= 0) 
		rez += ",font-strikeout";
	rez += QString(",bottom-offset=") + QString::number(ui.spinEditOffset->value()) + ",font-border=" + QString::number(ui.spinEditBorder->value());
	if (ui.rbhLeft->isChecked())
		rez += ",text-align=left";
	else if (ui.rbhRight->isChecked())
		rez += ",text-align=right";
	else
		rez += ",text-align=center";

	for (int i = 0; i < ui.trackLV->rowCount(); ++i)
	{
		if (ui.trackLV->item(i,0)->checkState() == Qt::Checked)
		{
			long iCodec = ui.trackLV->item(i,0)->data(Qt::UserRole).toLongLong();
			if (!iCodec) continue;
			QtvCodecInfo* codecInfo = (QtvCodecInfo*) (void*) iCodec;
			if (codecInfo->displayName == "H.264" || codecInfo->displayName == "VC-1" || codecInfo->displayName == "MPEG-2") {
				rez += QString(",video-width=") + QString::number(codecInfo->width);
				rez += QString(",video-height=") + QString::number(codecInfo->height);
				rez += QString(",fps=") + fpsTextToFpsStr(codecInfo->fpsText);
				return rez;
			}
		}
	}
	rez += ",video-width=1920,video-height=1080,fps=23.976";
	return rez;
}

QString TsMuxerWindow::getVideoMetaInfo(QtvCodecInfo* codecInfo)
{
	QString fpsStr;
	QString levelStr;
	QString rezStr = codecInfo->programName + ", ";
	for (int i = 0; i < codecInfo->fileList.size(); ++i) {
		if (i > 0)
			rezStr += '+';
		rezStr += QString("\"") + codecInfo->fileList[i] + "\"";
	}
	if (codecInfo->checkFPS)
		fpsStr = fpsTextToFpsStr(codecInfo->fpsText);
	if (codecInfo->checkLevel) 
		levelStr = QString::number(codecInfo->levelText.toDouble(), 'f', 1);

	if (ui.checkBoxCrop->isChecked())
		rezStr += QString(", ") + "restoreCrop";
	if (!fpsStr.isEmpty()) 
		rezStr += QString(", ") + "fps=" + fpsStr;
	if (!levelStr.isEmpty()) 
		rezStr += QString(", ") + "level=" + levelStr;
	if (codecInfo->addSEI) 
		rezStr += QString(", ") + "insertSEI";
	if (codecInfo->addSPS) 
		rezStr += QString(", ") + "contSPS";
	if (codecInfo->delPulldown == 1) 
		rezStr += QString(", ") + "delPulldown";
	if (codecInfo->arText != "Not change" && !codecInfo->arText.isEmpty()) 
		rezStr += QString(", ") + "ar=" + codecInfo->arText;
	return rezStr;
}

QString TsMuxerWindow::getAudioMetaInfo(QtvCodecInfo* codecInfo)
{
	QString rezStr = codecInfo->programName + ", ";
	for (int i = 0; i < codecInfo->fileList.size(); ++i) {
		if (i > 0) 
			rezStr += "+";
		rezStr  += QString("\"") + codecInfo->fileList[i] + "\"";
	}
	if (codecInfo->delay != 0) 
		rezStr += QString(", timeshift=") + QString::number(codecInfo->delay) + "ms";
	if (codecInfo->dtsDownconvert && codecInfo->programName == "A_DTS") 
		rezStr += ", down-to-dts";
	else if (codecInfo->dtsDownconvert && codecInfo->programName == "A_AC3") 
		rezStr += ", down-to-ac3";
	return rezStr;
}

void TsMuxerWindow::updateMetaLines()
{
	ui.memoMeta->clear();
	ui.memoMeta->append(getMuxOpts());
	QString tmpFps;	
	for (int i = 0; i < ui.trackLV->rowCount(); ++i) 
	{
		long iCodec = ui.trackLV->item(i,0)->data(Qt::UserRole).toLongLong();
		if (!iCodec) continue;
		QtvCodecInfo* codecInfo = (QtvCodecInfo*) (void*) iCodec;
		if (codecInfo->displayName == "H.264" || codecInfo->displayName == "VC-1" || codecInfo->displayName == "MPEG-2")
		{
			if (codecInfo->checkFPS) 
				tmpFps = fpsTextToFpsStr(codecInfo->fpsText);
			else
				tmpFps = codecInfo->fpsTextOrig;
			break;  
		}
	}

	QString prefix;
	QString postfix;
	for (int i = 0; i < ui.trackLV->rowCount(); ++i) 
	{
		if (ui.trackLV->item(i,0)->checkState() == Qt::Checked)
			prefix = "";
		else
			prefix = "#";
		long iCodec = ui.trackLV->item(i,0)->data(Qt::UserRole).toLongLong();
		if (!iCodec) continue;
		QtvCodecInfo* codecInfo = (QtvCodecInfo*) (void*) iCodec;

		postfix.clear();
		if (codecInfo->bindFps && codecInfo->displayName == "PGS" && !tmpFps.isEmpty())
			postfix += QString(", fps=") + tmpFps;
		if (codecInfo->displayName == "SRT") 
			postfix += getSrtParams();
		if (codecInfo->trackID != 0) 
			postfix += QString(", track=") + QString::number(codecInfo->trackID);
		if (!codecInfo->lang.isEmpty()) 
			postfix += QString(", lang=") + codecInfo->lang;
		if (!codecInfo->mplsFile.isEmpty()) 
			postfix += QString(", mplsFile=") + codecInfo->mplsFile;
		if (codecInfo->displayName == "H.264" || codecInfo->displayName == "VC-1" || codecInfo->displayName == "MPEG-2") 
			ui.memoMeta->append(prefix + getVideoMetaInfo(codecInfo) + postfix);
		else
			ui.memoMeta->append(prefix + getAudioMetaInfo(codecInfo) + postfix);
	}
}

void TsMuxerWindow::onFontBtnClicked()
{
	bool ok;
	QFont font;
	font.setFamily(ui.listViewFont->item(0,1)->text());
	font.setPointSize((ui.listViewFont->item(1,1)->text()).toInt());
	font.setItalic(ui.listViewFont->item(4,1)->text().indexOf("Italic") >= 0);
	font.setBold(ui.listViewFont->item(4,1)->text().indexOf("Bold") >= 0);
	font.setUnderline(ui.listViewFont->item(4,1)->text().indexOf("Underline") >= 0);
	font.setStrikeOut(ui.listViewFont->item(4,1)->text().indexOf("Strikeout") >= 0);
	font = QFontDialog::getFont(&ok, font, this);
	if (ok) 
	{
		//FT_Face fontFace = fontfreetypeFace();
		ui.listViewFont->item(0,1)->setText(font.family());
		ui.listViewFont->item(1,1)->setText(QString::number(font.pointSize()));
		QString optStr;
		if (font.italic())
			optStr += "Italic";
		if (font.bold()) {
			if (!optStr.isEmpty()) optStr += ',';
			optStr += "Bold";
		}
		if (font.underline()) {
			if (!optStr.isEmpty()) optStr += ',';
			optStr += "Underline";
		}
		if (font.strikeOut()) {
			if (!optStr.isEmpty()) optStr += ',';
			optStr += "Strikeout";
		}
		ui.listViewFont->item(4,1)->setText(optStr);
	    updateMetaLines();
	}
}

quint32 bswap(quint32 val)
{
	return ((val & 0xff0000u >> 16)) + (val & 0xff00u) + ((val & 0xffu) << 16);
}

int colorLight(QColor color)
{
	return (0.257 * color.red()) + (0.504 * color.green()) + (0.098 * color.blue()) + 16;
}

void TsMuxerWindow::onColorBtnClicked()
{
	bool ok;
	QColor color = bswap(ui.listViewFont->item(2,1)->text().toLongLong(&ok, 16));
	color = QColorDialog::getColor(color, this);
	QString str = QString::number(bswap(color.rgb()), 16);
	while (str.length() < 8)
		str = '0' + str;
	QTableWidgetItem* item = ui.listViewFont->item(2,1);
	item->setBackground(QBrush(color));
	if (colorLight(color) < 128)
		item->setForeground(QBrush(QColor(255,255,255,255)));
	else
		item->setForeground(QBrush(QColor(0,0,0,255)));
	item->setText(QString("0x") + str);
    updateMetaLines();
}

void TsMuxerWindow::onGeneralCheckboxClicked()
{
	ui.editMaxBitrate->setEnabled(ui.checkBoxRVBR->isChecked());
	ui.editMinBitrate->setEnabled(ui.checkBoxRVBR->isChecked());
	ui.editCBRBitrate->setEnabled(ui.checkBoxCBR->isChecked());
	ui.BlackplaylistCombo->setEnabled(ui.checkBoxBlankPL->isChecked());
	ui.BlackplaylistLabel->setEnabled(ui.checkBoxBlankPL->isChecked());
	updateMetaLines();
}

void TsMuxerWindow::onGeneralSpinboxValueChanged()
{
	updateMetaLines();
}

void TsMuxerWindow::onChapterParamsChanged()
{
	ui.memoChapters->setEnabled(ui.radioButtonCustomChapters->isChecked());
	ui.spinEditChapterLen->setEnabled(ui.radioButtonAutoChapter->isChecked());
	updateMetaLines();
}

void TsMuxerWindow::onSplitCutParamsChanged()
{
	ui.spinEditSplitDuration->setEnabled(ui.splitByDuration->isChecked());
	ui.labelSplitByDur->setEnabled(ui.splitByDuration->isChecked());
	ui.editSplitSize->setEnabled(ui.splitBySize->isChecked());
	ui.comboBoxMeasure->setEnabled(ui.splitBySize->isChecked());

	ui.editCutStart->setEnabled(ui.checkBoxCut->isChecked());
	ui.editCutEnd->setEnabled(ui.checkBoxCut->isChecked());
	ui.measureCutStart->setEnabled(ui.checkBoxCut->isChecked());
	ui.measureCutEnd->setEnabled(ui.checkBoxCut->isChecked());
	updateMetaLines();
}

void TsMuxerWindow::onFontParamsChanged()
{
	updateMetaLines();
}


void TsMuxerWindow::onRemoveBtnClick()
{
	if (!ui.inputFilesLV->currentItem())
		return;
	int idx = ui.inputFilesLV->currentRow();
	bool needDelAddTracks = false;

	if (ui.inputFilesLV->currentItem()->text().left(4) != " ++ ")
	{
		if (idx < ui.inputFilesLV->count()-1)
		{
			if (ui.inputFilesLV->currentItem()->data(Qt::UserRole).toInt() == 1)
				needDelAddTracks = true;
			else { 
				QString text = ui.inputFilesLV->item(idx+1)->text();
				if (text.length() >= 4  && text.left(4) == " ++ ") 
					ui.inputFilesLV->item(idx+1)->setText(text.mid(4));
			}
		}
	}
	delTracksByFileName(myUnquoteStr(ui.inputFilesLV->currentItem()->text()));
	ui.inputFilesLV->takeItem(idx);
	if (idx >= ui.inputFilesLV->count())
		idx--;
	if (needDelAddTracks)
	{
		while (idx < ui.inputFilesLV->count())
		{
			QString text = ui.inputFilesLV->item(idx)->text();
			if (text.length() >= 4 && text.left(4) == " ++ ")
			{
				delTracksByFileName(myUnquoteStr(ui.inputFilesLV->item(idx)->text()));
				ui.inputFilesLV->takeItem(idx);
			}
			else break;
		}
	}
	if (ui.inputFilesLV->count() > 0) 
		ui.inputFilesLV->setCurrentRow(idx);
}

void TsMuxerWindow::delTracksByFileName(const QString& fileName)
{
    for (int i = ui.trackLV->rowCount()-1; i >= 0; --i)
	{
		long iCodec = ui.trackLV->item(i, 0)->data(Qt::UserRole).toLongLong();
		if (iCodec) {
			QtvCodecInfo* info = (QtvCodecInfo*) (void*) iCodec;
			for (int j = info->fileList.count()-1; j >= 0; --j)
			{
				if (info->fileList[j] == fileName) {
					info->fileList.removeAt(j);
					break;
				}
			}
			if (info->fileList.count() == 0)
				deleteTrack(i);
		}
	}
    updateMetaLines();
}

void TsMuxerWindow::deleteTrack(int idx)
{
	trackLVIngoreOnChange = true;
	long iCodec = ui.trackLV->item(idx, 0)->data(Qt::UserRole).toLongLong();
	delete (QtvCodecInfo*) (void*) iCodec;
    int lastItemIndex = idx; //trackLV.items[idx].index;
	ui.trackLV->removeRow(idx);
	if (ui.trackLV->rowCount() == 0) 
	{
 		while(ui.tabWidgetTracks->count())
			ui.tabWidgetTracks->removeTab(0);
		ui.tabWidgetTracks->addTab(ui.tabSheetFake, TI_DEFAULT_TAB_NAME);
		if (ui.radioButtonTS->isChecked())
			ui.outFileName->setText("default.ts");
		else if (ui.radioButtonM2TS->isChecked())
			ui.outFileName->setText("default.m2ts");
		else
			ui.outFileName->text().clear();
		outFileNameModified = false;
	}
	else {
		if (lastItemIndex >  ui.trackLV->rowCount()-1) 
			--lastItemIndex;
		if (lastItemIndex >= 0) {
			ui.trackLV->setCurrentCell(lastItemIndex,0);
			ui.trackLV->setRangeSelected(QTableWidgetSelectionRange(lastItemIndex, 0, lastItemIndex, 4), true);
		}
		updateNum();
	}
    updateMetaLines();
    ui.moveupBtn->setEnabled(ui.trackLV->currentItem() != 0);
    ui.movedownBtn->setEnabled(ui.trackLV->currentItem() != 0);
    ui.removeTrackBtn->setEnabled(ui.trackLV->currentItem() != 0);
	trackLVIngoreOnChange = false;
	trackLVItemSelectionChanged();
}

void TsMuxerWindow::updateNum()
{
	//for (int i = 0; i < ui.trackLV->rowCount(); ++i)
	//	ui.trackLV->item(i,0)->setText(QString::number(i+1));
}

void TsMuxerWindow::onAppendButtonClick()
{
	QList<QtvCodecInfo> codecList;
	if (ui.inputFilesLV->currentItem() == 0)
	{
		QMessageBox msgBox(this);
		msgBox. setWindowTitle("No track selected");
		msgBox.setText("No track selected");
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.exec();
		return;
	}
	QString fileName = QDir::toNativeSeparators(QFileDialog::getOpenFileName(this, tr("Append media file"), lastSelectedDir, tr(fileDialogFilter)));
	if (fileName.isEmpty())
		return;
	lastSelectedDir = fileName;
	addFileList.clear();
	addFileList << QUrl::fromLocalFile(fileName);
	appendFile();
}

void TsMuxerWindow::appendFile()
{
	if (addFileList.isEmpty())
		return;
	newFileName = QDir::toNativeSeparators(addFileList[0].toLocalFile());
	addFileList.removeAt(0);
	if (!checkFileDuplicate(newFileName)) 
		return;
	//disconnect(this, SIGNAL(tsMuxerSuccessFinished()));
	//disconnect(this, SIGNAL(codecListReady()));
	disconnect();
	connect(this, SIGNAL(tsMuxerSuccessFinished()), this, SLOT(getCodecInfo()));
	connect(this, SIGNAL(codecListReady()), this, SLOT(continueAppendFile()));
	connect(this, SIGNAL(fileAppended()), this, SLOT(appendFile()));
	runInMuxMode = false;
	shellExecute(QDir::toNativeSeparators(QCoreApplication::applicationDirPath()) + QDir::separator() + "tsMuxeR", QStringList() << newFileName);
}

void TsMuxerWindow::continueAppendFile()
{
	QString parentFileName = myUnquoteStr((ui.inputFilesLV->currentItem()->text()));
	QFileInfo newFi(unquoteStr(newFileName));
	QFileInfo oldFi(unquoteStr(parentFileName));
	if (newFi.suffix().toUpper() != oldFi.suffix().toUpper())
	{
		QMessageBox msgBox(this);
		msgBox. setWindowTitle("Invalid file extension");
		msgBox.setText("Appended file must have same file extension.");
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.exec();
		return;
	}
	trackLVIngoreOnChange = true;
	doAppendInt(newFileName, parentFileName);
	updateMetaLines();
	if (!outFileNameModified) 
	{
		modifyOutFileName(newFileName);
		outFileNameModified = true;
	}
	trackLVIngoreOnChange = false;
	emit fileAppended();
}

void TsMuxerWindow::onRemoveTrackButtonClick()
{
	if (ui.trackLV->currentItem())
		deleteTrack(ui.trackLV->currentRow());
}

void TsMuxerWindow::onMoveUpButtonCLick()
{
	if (ui.trackLV->currentItem() == 0 || ui.trackLV->currentRow() < 1) 
        return;
	trackLVIngoreOnChange = true;
	moveRow(ui.trackLV->currentRow(), ui.trackLV->currentRow()-1);
	updateMetaLines();
	updateNum();
	trackLVIngoreOnChange = false;
}

void TsMuxerWindow::onMoveDownButtonCLick()
{
	if (ui.trackLV->currentItem() == 0 || ui.trackLV->rowCount() == 0 || ui.trackLV->currentRow() == ui.trackLV->rowCount()-1) 
        return;
	trackLVIngoreOnChange = true;
	moveRow(ui.trackLV->currentRow(), ui.trackLV->currentRow()+2);
	updateMetaLines();
	updateNum();
	trackLVIngoreOnChange = false;
}

void TsMuxerWindow::moveRow(int index, int index2)
{
	ui.trackLV->insertRow(index2);
	ui.trackLV->setRowHeight(index2, 18);
	if (index2 < index)
		index++;
	for (int i = 0; i < ui.trackLV->columnCount(); ++i)
		ui.trackLV->setItem(index2, i, ui.trackLV->item(index,i)->clone());
	ui.trackLV->removeRow(index);
	if (index2 > index)
		index2--;
	ui.trackLV->setRangeSelected(QTableWidgetSelectionRange(index2, 0, index2, 4), true);
	ui.trackLV->setCurrentCell(index2, 0);
}

void TsMuxerWindow::RadioButtonMuxClick()
{
	if (outFileNameDisableChange)
		return;
	if (ui.radioButtonDemux->isChecked())
		ui.buttonMux->setText("Sta&rt demuxing");
	else
		ui.buttonMux->setText("Sta&rt muxing");
	outFileNameDisableChange = true;
	if (ui.radioButtonBluRay->isChecked() || ui.radioButtonDemux->isChecked() || ui.radioButtonAVCHD->isChecked())
	{
		QFileInfo fi(unquoteStr(ui.outFileName->text()));
		if (!fi.suffix().isEmpty()) 
		{
			oldFileName = fi.fileName();
			ui.outFileName->setText(QDir::toNativeSeparators(fi.absolutePath()) + QDir::separator());
		}
	}
	else {
		if (!oldFileName.isEmpty()) 
		{
			ui.outFileName->setText(QDir::toNativeSeparators(ui.outFileName->text()));
			if (!ui.outFileName->text().isEmpty() && ui.outFileName->text().right(1) != QDir::separator())
				ui.outFileName->setText(ui.outFileName->text() + QDir::separator());
			ui.outFileName->setText(ui.outFileName->text() + oldFileName);
			oldFileName.clear();
		}
		if (ui.radioButtonTS->isChecked()) {
			ui.outFileName->setText(changeFileExt(ui.outFileName->text(), "ts"));
			saveDialogFilter = TS_SAVE_DIALOG_FILTER;
		}
		else {
			ui.outFileName->setText(changeFileExt(ui.outFileName->text(), "m2ts"));
			saveDialogFilter = M2TS_SAVE_DIALOG_FILTER;
		}
	}
	ui.editDelay->setEnabled(!ui.radioButtonDemux->isChecked());
	updateMetaLines();
	outFileNameDisableChange = false;
}

void TsMuxerWindow::outFileNameChanged()
{
	outFileNameModified = true;
	if (outFileNameDisableChange)
		return;
	if (ui.radioButtonDemux->isChecked() || ui.radioButtonBluRay->isChecked() || ui.radioButtonAVCHD->isChecked())
		return;
	outFileNameDisableChange = true;
	QFileInfo fi(unquoteStr(ui.outFileName->text().trimmed()));
	if (fi.suffix().toUpper() == "M2TS" || fi.suffix().toUpper() == "M2TS\"")
		ui.radioButtonM2TS->setChecked(true);
	else
		ui.radioButtonTS->setChecked(true);
	outFileNameDisableChange = false;
}

void TsMuxerWindow::saveFileDialog()
{
	if (ui.radioButtonDemux->isChecked() || ui.radioButtonBluRay->isChecked() || ui.radioButtonAVCHD->isChecked())
	{
		QString folder = QDir::toNativeSeparators(QFileDialog::getExistingDirectory(this, lastSelectedDir));
		if (!folder.isEmpty()) {
			ui.outFileName->setText(folder + QDir::separator());
			outFileNameModified = true;
			lastSelectedDir = folder;
		}
	}
	else {
		QFileInfo fi(unquoteStr(ui.outFileName->text()));
		QString fileName = QDir::toNativeSeparators(QFileDialog::getSaveFileName(this, "Select file for muxing", fi.absolutePath(), saveDialogFilter));
		if (!fileName.isEmpty())
			ui.outFileName->setText(fileName);
	}
}

void TsMuxerWindow::startMuxing()
{
	ui.outFileName->setText(unquoteStr(ui.outFileName->text().trimmed()));
	if (ui.radioButtonM2TS->isChecked()) 
	{
		QFileInfo fi(ui.outFileName->text());
		if (fi.suffix().toUpper() != "M2TS") {
			QMessageBox msgBox(this);
			msgBox. setWindowTitle("Invalid file name");
			msgBox.setText(QString("The output file \"") + ui.outFileName->text() + "\" has invalid extension. Please, change file extension to \".m2ts\"");
			msgBox.setIcon(QMessageBox::Warning);
			msgBox.setStandardButtons(QMessageBox::Ok);
			msgBox.exec();
			return;
		}
	}
	bool isFile = ui.radioButtonM2TS->isChecked() || ui.radioButtonTS->isChecked();
	if (isFile && QFile::exists(ui.outFileName->text()))
	{
		QString fileOrDir = isFile ? "file" : "directory";
		QMessageBox msgBox(this);
		msgBox. setWindowTitle(QString("Overwrite existing ") + fileOrDir +"?");
		msgBox.setText(QString("The output ") + fileOrDir + " \"" + ui.outFileName->text() + "\" already exists. Do you want to overwrite it?\"");
		msgBox.setIcon(QMessageBox::Question);
		msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
		if (msgBox.exec() != QMessageBox::Yes)
			return;
	}

	QFileInfo fi(ui.outFileName->text());
	metaName = QDir::toNativeSeparators(QDir::tempPath()) + QDir::separator() +  "tsMuxeR_" + fi.completeBaseName () + ".meta";
	metaName = metaName.replace(' ', '_');
	if (!saveMetaFile(metaName)) {
		metaName.clear();
		return;
	}
	muxForm.prepare(!ui.radioButtonDemux->isChecked() ? "Muxing in progress" : "Demuxing in progress");
	ui.buttonMux->setEnabled(false);
	ui.addBtn->setEnabled(false);
	ui.btnAppend->setEnabled(false);
	muxForm.show();
	disconnect();
	//QCoreApplication::dir
	runInMuxMode = true;
	shellExecute(QDir::toNativeSeparators(QCoreApplication::applicationDirPath()) + QDir::separator() + "tsMuxeR", QStringList() << metaName << quoteStr(ui.outFileName->text()));
}

void TsMuxerWindow::saveMetaFileBtnClick()
{
	QString metaName = QFileDialog::getSaveFileName(this, "", changeFileExt(ui.outFileName->text(), "meta"), saveMetaFilter);
	if (metaName.isEmpty())
		return;
	QFileInfo fi(metaName);
	QDir dir;
	dir.mkpath(fi.absolutePath());
	saveMetaFile(metaName);
}

bool TsMuxerWindow::saveMetaFile(const QString& metaName)
{
	QFile file(metaName);
	if (!file.open(QIODevice::WriteOnly)) {
		QMessageBox msgBox(this);
		msgBox. setWindowTitle("Can't create temporary meta file");
		msgBox.setText(QString("Can't create temporary meta file \"") + metaName + "\"");
		msgBox.setIcon(QMessageBox::Warning);
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.exec();
		return false;
	}
	QByteArray metaText = ui.memoMeta->toPlainText().toLocal8Bit();
	file.write(metaText);
	file.close();
	return true;
}

void TsMuxerWindow::closeEvent(QCloseEvent * event )
{
	if (!metaName.isEmpty()) {
		QFile::remove(metaName);
		metaName.clear();
	}
	muxForm.close();
	event->accept();
}

void TsMuxerWindow::dragEnterEvent(QDragEnterEvent *event) 
{ 
	if (event->mimeData()->hasFormat("text/plain") || event->mimeData()->hasFormat("text/uri-list"))
	{
		if (ui.addBtn->isEnabled()) 
		{
			opacityTimer.stop();
			setWindowOpacity(0.9);
			event->acceptProposedAction();
			QWidget* w = childAt(event->pos());
			updateBtns(w);
		}
	}
}
 
void TsMuxerWindow::dropEvent(QDropEvent* event)
{
	setWindowOpacity(1.0);
	updateBtns(0);
	QWidget* w = childAt(event->pos());
	QString wName;
	if (w)
		wName = w->objectName();
	if (event->mimeData()->hasFormat("text/uri-list")) {
		addFileList = event->mimeData()->urls();
		event->acceptProposedAction();
	}
	else if (event->mimeData()->hasFormat("text/plain"))
	{
		QList<QString> strList;
		addFileList.clear();
		splitLines(event->mimeData()->text(), strList);
		QList <QUrl> urls = event->mimeData()->urls();
		for (int i = 0; i < strList.size(); ++i)
			addFileList << QUrl::fromLocalFile(strList[i]);
		event->acceptProposedAction();
	}
	if (addFileList.isEmpty())
		return;
	if (wName == "btnAppend" && ui.btnAppend->isEnabled())
		appendFile();
	else if (ui.addBtn->isEnabled()) 
		addFile();
}

void TsMuxerWindow::dragMoveEvent(QDragMoveEvent *event)
{
    event->acceptProposedAction();
	QWidget* w = childAt(event->pos());
	updateBtns(w);
}

void TsMuxerWindow::updateBtns(QWidget* w)
{
	if (w) {
		QString wName = w->objectName();
		ui.btnAppend->setDefault(wName == "btnAppend" && ui.btnAppend->isEnabled());
		ui.addBtn->setDefault(wName == "addBtn" && ui.addBtn->isEnabled());
	}
	else {
		ui.btnAppend->setDefault(false);
		ui.addBtn->setDefault(false);
	}
	QFont font = ui.removeFile->font();
	QFont bFont(font);
	bFont.setBold(true);
	if (ui.btnAppend->isDefault())
		ui.btnAppend->setFont(bFont);
	else
		ui.btnAppend->setFont(font);
	if (ui.addBtn->isDefault())
		ui.addBtn->setFont(bFont);
	else
		ui.addBtn->setFont(font);
}

void TsMuxerWindow::dragLeaveEvent ( QDragLeaveEvent * event)  
{
	opacityTimer.start(100);
}

void TsMuxerWindow::onOpacityTimer()  
{
	opacityTimer.stop();
	setWindowOpacity(1.0);
	updateBtns(0);
}
